#ifndef MODULESINTERFACE_H
#define MODULESINTERFACE_H

#include "Event.h"

namespace SANCmodulesVar {

struct double_complex { double re, im; };

void setMomenta(SANC::Event & event);

extern "C" {


extern struct Masses {
    double ma,mv,mh,mz,mw,men,mel,mup,mdn,mmn,mmo,mch,mst,mtn,mta,mtp,mbt;
} masses_;

extern struct R2masses {
    double ram2,rvm2,rhm2,rzm2,rwm2,renm2,relm2,rupm2,rdnm2,rmnm2,rmom2,rchm2,rstm2,rtnm2,rtam2,
    rtpm2,rbtm2;
} r2masses_;

extern struct C2masses{
    double_complex cm2,vm2,hm2,zm2,wm2,enm2,elm2,upm2,dnm2,mnm2,mom2,chm2,stm2,tnm2,tam2,tpm2,
    btm2;
} c2masses_;

extern struct Cstw {
    double ctw,stw,ctw2,stw2,ctw4,stw4,ctw6,stw6;
} cstw_;

extern struct Rmhwz {
    double rwz,rhz,rhw,renh,relh,ruph,rdnh,rmnh,rmoh,rchh,rsth,rtnh,rtah,rtph,rbth,renz,relz,
    rupz,rdnz,rmnz,rmoz,rchz,rstz,rtnz,rtaz,rtpz,rbtz,renw,relw,rupw,rdnw,rmnw,rmow,rchw,rstw,
    rtnw,rtaw,rtpw,rbtw;
} rmhwz_;

extern struct Qavpm {
    double qen,qel,qup,qdn,qmn,qmo,qch,qst,qtn,qta,qtp,qbt,aen,ael,aup,adn,amn,amo,ach,ast,atn,
    ata,atp,abt,ven,vel,vup,vdn,vmn,vmo,vch,vst,vtn,vta,vtp,vbt,vpaen,vmaen,vpael,vmael,vpaup,
    vmaup,vpadn,vmadn,vpamn,vmamn,vpamo,vmamo,vpach,vmach,vpast,vmast,vpatn,vmatn,vpata,vmata,
    vpatp,vmatp,vpabt,vmabt,dfen,dfel,dfup,dfdn,dfmn,dfmo,dfch,dfst,dftn,dfta,dftp,dfbt;
} qavpm_;

extern struct Flags {
    int iborn,iqed,iew,iqcd,gfscheme,ilin,ifgg,its,irun,alr;
} flags_;

extern struct Qfi3f {
    double qf[12],i3f[12],rmf1[12];
} qfi3f_;

extern struct Nc {
    double nc,fc,cfl,cfq,cfscheme,cfprime;
} nc_;

extern struct Width {
    double wz,ww,wh,wtp,gz,gw,gh,gtp;
} width_;

extern struct Widthm {
    double_complex mz2,mw2,mh2,mtp2,mz2c,mw2c,mh2c,mtp2c;
} widthm_;

extern struct Consts {
    double alpha,alphai,gf,g,e,alphas,gs,cf,conhc,pi,sr2,deltaa;
} consts_;

extern struct Consti {
    double_complex i_;
} consti_;

extern struct Scales {
    double thmu2,tlmu2,omega;
} scales_;

extern struct Polar {
    double lamep,lamem;
} polar_;

extern struct Momenta {
    double pg1[5],pg2[5],pg3[5],pg4[5],pg5[5];
} momenta_;

} // extern C

} // namespace

namespace SANCmodules {


extern "C" void nc_br_12_12__12_12_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_br_12_12__02_04_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_br_12_12__16_16_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_br_12_12__20_20_(double*, double*, double*, double*, double*, double*);

extern "C" void nc_si_12_12__12_12_(double*, double*, double*, double*);
extern "C" void nc_si_12_12__02_04_(double*, double*, double*, double*);
extern "C" void nc_si_12_12__16_16_(double*, double*, double*, double*);
extern "C" void nc_si_12_12__20_20_(double*, double*, double*, double*);

extern "C" void nc_ha_12_12__12_12_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_ha_12_12__02_04_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_ha_12_12__16_16_(double*, double*, double*, double*, double*, double*);
extern "C" void nc_ha_12_12__20_20_(double*, double*, double*, double*, double*, double*);
}

namespace SANC {

class Info;

class ModulesInterface
{
public:
    ModulesInterface(Info & _info);
    void init();
private:
    Info & info;
};

}

#endif // MODULESINTERFACE_H
