************************************************************************
* This is the FORTRAN module (nc_ha_12_12__20_20) to calculate differential
* hard photon Bremsstrahlung for the el^+ + el^- -> tau^+ + tau^- process.
*
* Copyright (C), SANC Project Team, 2002-2021.
*
* http://sanc.jinr.ru/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine nc_ha_12_12__20_20_isrSA(p1,p2,p3,p4,p5,Ql,DZZ,amp)
      implicit none!
      include 's2n_declare.h'
      real*8 p1(5), p2(5), p3(5), p4(5), p5(5)
      complex*16 DZZ(2, 2)
      real*8 m1, m2, m3, m4
      complex*16 amp(2, 2, 2, 2, 2)
      real*8 Pair
      complex*16 SpAA, SpBB
      integer*4 h1, h2, h3, h4, h5, h12, j1, j2, i

      integer*4 a, b, o
      real*8 Ql, Ql_2sqrt2
      real*8 p12(5), p34(5)
      real*8 k5(5), k1(5), k2(5), k3(5), k4(5)
      complex*16 A15A, A25A, A35A, A45A
      complex*16 B51B, B52B, B53B, B54B
      complex*16 A12A, A13A, A14A, A23A, A24A, A34A
      complex*16 B21B, B31B, B32B,  B41B, B42B, B43B
      complex*16 inv_A15A, inv_A25A, inv_A35A, inv_A45A
      complex*16 inv_B51B, inv_B52B, inv_B53B, inv_B54B
      complex*16 DLL, DRL, DLR, DRR
      complex*16 varsigma1, varsigma2, varsigma3, varsigma4
      complex*16 varsigma1c, varsigma2c, varsigma3c, varsigma4c

      complex*16 Sfact(2),Bfact(2,2,2,2),C1fact(2,2),C2fact(2,2),G1fact(2,2,2,2),G2fact(2,2,2,2)
      complex*16 Avb1vb3A(2,2),Avb5vb3A(2,2),Bvb1vb3B(2,2),Bvb5vb3B(2,2)
      complex*16 Avb1u4A(2,2),Avb5u4A(2,2),Bvb1u4B(2,2),Bvb5u4B(2,2)
      complex*16 Avb3u2A(2,2),Avb3u5A(2,2),Bvb3u2B(2,2),Bvb3u5B(2,2)
      complex*16 Au4u2A(2,2),Au4u5A(2,2),Bu4u2B(2,2),Bu4u5B(2,2)
!-----------------------------------------------------------------------
      m1 = p1(5)
      m2 = p2(5)
      m3 = p3(5)
      m4 = p4(5)
      Ql_2sqrt2 = 2*sqrt(2d0)*Ql

      a = 1; ! '+'
      b = 2; ! '-'
      o = 3; ! '0'
      call S_Assoc1(p5, k5);
      call S_Assoc(p1, p5, k1);
      call S_Assoc(p2, p5, k2);
      call S_Assoc(p3, p5, k3);
      call S_Assoc(p4, p5, k4);

      DLL= DZZ(2,2);
      DRL= DZZ(1,2);
      DLR= DZZ(2,1);
      DRR= DZZ(1,1);

      A15A = SpAA(p1, k5);
      A25A = SpAA(p2, k5);
      A35A = SpAA(p3, k5);
      A45A = SpAA(p4, k5);
      A12A = SpAA(k1, k2);
      A13A = SpAA(k1, k3);
      A14A = SpAA(k1, k4);
      A23A = SpAA(k2, k3);
      A24A = SpAA(k2, k4);
      A34A = SpAA(k3, k4);

      B51B = SpBB(k5, p1);
      B52B = SpBB(k5, p2);
      B53B = SpBB(k5, p3);
      B54B = SpBB(k5, p4);
      B21B = SpBB(k2, k1);
      B31B = SpBB(k3, k1);
      B32B = SpBB(k3, k2);
      B41B = SpBB(k4, k1);
      B42B = SpBB(k4, k2);
      B43B = SpBB(k4, k3);

      inv_A15A = 1.d0 / A15A;
      inv_A25A = 1.d0 / A25A;
      inv_A35A = 1.d0 / A35A;
      inv_A45A = 1.d0 / A45A;
      inv_B51B = 1.d0 / B51B;
      inv_B52B = 1.d0 / B52B;
      inv_B53B = 1.d0 / B53B;
      inv_B54B = 1.d0 / B54B;

      varsigma1 = m1*inv_A15A
      varsigma2 = m2*inv_A25A
      varsigma3 = m3*inv_A35A
      varsigma4 = m4*inv_A45A

      varsigma1c = m1*inv_B51B
      varsigma2c = m2*inv_B52B
      varsigma3c = m3*inv_B53B
      varsigma4c = m4*inv_B54B

!-----------------------------------------------------------------------
      Sfact(a) = +B21B*inv_B51B*inv_B52B
      Sfact(b) = -A12A*inv_A15A*inv_A25A

      C1fact(a,a) =  inv_B51B
      C1fact(b,b) = -inv_A15A
      C1fact(a,b) =  0d0
      C1fact(b,a) =  0d0

      C2fact(a,a) = -inv_B52B
      C2fact(b,b) =  inv_A25A
      C2fact(a,b) =  0d0
      C2fact(b,a) =  0d0

      Avb1vb3A(a,a) =  A13A
      Avb1vb3A(a,b) =  A15A*varsigma3
      Avb1vb3A(b,a) = -A35A*varsigma1
      Avb1vb3A(b,b) =  0D0

      Avb5vb3A(a,a) = -A35A
      Avb5vb3A(a,b) =  0d0
      Avb5vb3A(b,a) =  0d0
      Avb5vb3A(b,b) =  0D0

      Bvb1vb3B(a,a) =  0D0
      Bvb1vb3B(a,b) = -B53B*varsigma1c
      Bvb1vb3B(b,a) =  B51B*varsigma3c
      Bvb1vb3B(b,b) = -B31B

      Bvb5vb3B(a,a) = 0D0
      Bvb5vb3B(a,b) = 0D0
      Bvb5vb3B(b,a) = 0D0
      Bvb5vb3B(b,b) = B53B

      Avb1u4A(a,a) =  A14A
      Avb1u4A(a,b) =  A15A*varsigma4
      Avb1u4A(b,a) = -A45A*varsigma1
      Avb1u4A(b,b) =  0D0

      Avb5u4A(a,a) = -A45A
      Avb5u4A(a,b) =  0D0
      Avb5u4A(b,a) =  0D0
      Avb5u4A(b,b) =  0D0

      Bvb1u4B(a,a) =  0D0
      Bvb1u4B(a,b) = -B54B*varsigma1c
      Bvb1u4B(b,a) =  B51B*varsigma4c
      Bvb1u4B(b,b) = -B41B

      Bvb5u4B(a,a) = 0D0
      Bvb5u4B(a,b) = 0D0
      Bvb5u4B(b,a) = 0D0
      Bvb5u4B(b,b) = B54B

      Avb3u2A(a,a) = -A23A
      Avb3u2A(a,b) =  A35A*varsigma2
      Avb3u2A(b,a) = -A25A*varsigma3
      Avb3u2A(b,b) =  0D0

      Avb3u5A(a,a) = A35A
      Avb3u5A(a,b) = 0D0
      Avb3u5A(b,a) = 0D0
      Avb3u5A(b,b) = 0D0

      Bvb3u2B(a,a) =  0D0
      Bvb3u2B(a,b) = -B52B*varsigma3c
      Bvb3u2B(b,a) =  B53B*varsigma2c
      Bvb3u2B(b,b) =  B32B

      Bvb3u5B(a,a) =  0D0
      Bvb3u5B(a,b) =  0D0
      Bvb3u5B(b,a) =  0D0
      Bvb3u5B(b,b) = -B53B

      Au4u2A(a,a) = -A24A
      Au4u2A(a,b) =  A45A*varsigma2
      Au4u2A(b,a) = -A25A*varsigma4
      Au4u2A(b,b) =  0D0

      Au4u5A(a,a) = A45A
      Au4u5A(a,b) = 0D0
      Au4u5A(b,a) = 0D0
      Au4u5A(b,b) = 0D0

      Bu4u2B(a,a) =  0D0
      Bu4u2B(a,b) = -B52B*varsigma4c
      Bu4u2B(b,a) =  B54B*varsigma2c
      Bu4u2B(b,b) =  B42B

      Bu4u5B(a,a) =  0D0
      Bu4u5B(a,b) =  0D0
      Bu4u5B(b,a) =  0D0
      Bu4u5B(b,b) = -B54B

      do h1=1,2
        do h2=1,2
          do h3 =1,2
            do h4 = 1,2
              Bfact(h1,h2,h3,h4) =   Bvb1vb3B(h1,h3)*DRR*Au4u2A(h4,h2) +
     &                               Bvb1u4B(h1,h4)*DRL*Avb3u2A(h3,h2) +
     &                               Avb1u4A(h1,h4)*DLR*Bvb3u2B(h3,h2) +
     &                               Avb1vb3A(h1,h3)*DLL*Bu4u2B(h4,h2);
            enddo
          enddo
        enddo
      enddo

      do h12=1,2
        do h3 =1,2
          do h4 = 1,2
            G1fact(h12,h3,h4,a) =   Bvb1vb3B(h12,h3)*DRR*Au4u5A(h4,a) +
     &                              Bvb1u4B(h12,h4)*DRL*Avb3u5A(h3,a);
            G1fact(h12,h3,h4,b) =   Avb1u4A(h12,h4)*DLR*Bvb3u5B(h3,b) +
     &                              Avb1vb3A(h12,h3)*DLL*Bu4u5B(h4,b);
            G2fact(h12,h3,h4,a) =   Avb5u4A(a,h4)*DLR*Bvb3u2B(h3,h12) +
     &                              Avb5vb3A(a,h3)*DLL*Bu4u2B(h4,h12);
            G2fact(h12,h3,h4,b) =   Bvb5vb3B(b,h3)*DRR*Au4u2A(h4,h12) +
     &                              Bvb5u4B(b,h4)*DRL*Avb3u2A(h3,h12);
          enddo
        enddo
      enddo

      do h1=1,2
        do h2=1,2
          do h3 =1,2
            do h4 = 1,2
              do h5 = 1,2
                amp(h1,h2,h3,h4,h5) = Ql_2sqrt2*(Sfact(h5)*Bfact(h1,h2,h3,h4)-C1fact(h1,h5)*G2fact(h2,h3,h4,h5)-C2fact(h2,h5)*G1fact(h1,h3,h4,h5))
              enddo
            enddo
          enddo
        enddo
      enddo

      return
      end

      subroutine nc_ha_12_12__20_20(s,spr,ph4,costh4,costh5,hard)
      implicit none!
      include 's2n_declare.h'
      real*8 ph4
      integer*4 h1,h2,h3,h4,h5,j1,j2,i
      real*8 sqs,sqspr
      real*8 costh4,costh5
      complex*16  hel_amp(2,2,2,2,2), ampISR(2,2,2,2,2), ampFSR(2,2,2,2,2)
      real*8 amp2(2,2,2,2,2),hard2(2,2)
      real*8 betai
      real*8 p3r1(5),p4r1(5),p3r2(5),p4r2(5)
      real*8 M_I,M_F

      complex*16 DZZisr(2,2), DZZfsr(2,2)
      complex*16 DLL2_3, DRR2_3, DRL2_3, DRL3_2
      complex*16 cc1(2,2), cc2(2,2)
      real*8 p1(5), p2(5), p3(5), p4(5), p5(5);
      real*8 pg1(5),pg2(5),pg3(5),pg4(5),pg5(5)
      common/momenta/pg1,pg2,pg3,pg4,pg5

#ifndef GENERATOR
      real*8 mp(5)

      mp = (/mel, mel, mta, mta, 0d0/)
      ! toMomenta for cos(theta) & toMomenta1 for rapidity
      call toMomenta(s,spr,ph4,costh4,costh5,mp)
#endif

!-----------------------------------------------------------------------
!--- p(1) = px, p(2) = py, p(3) = pz, p(4) = e, p(5) = m ---------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!--- 4-momenta of particles in the laboratory reference frame as input
      p1 = pg1
      p2 = pg2
      p3 = pg3
      p4 = pg4
      p5 = pg5
!--- But we work with all momenta to zero setup
!--- Do all momenta incoming
      do i = 1,4
        p3(i) = -p3(i)
        p4(i) = -p4(i)
        p5(i) = -p5(i)
      enddo

      M_I = p1(5)
      M_F = p3(5)
      sqs = sqrt(s)
      sqspr = sqrt(spr)
      betai = sqrt((sqs-2d0*M_I)*(sqs+2d0*M_I))/sqs
      betaf = sqrt((sqspr-2d0*M_F)*(sqspr+2d0*M_F))/sqspr

      call S_CC(p1,p5,-1d0,cc1)
      call S_CC(p2,p5,+1d0,cc2)

      DZZisr(2,2)=DLL2_3(spr);
      DZZisr(1,2)=DRL2_3(spr);
      DZZisr(2,1)=DRL3_2(spr);
      DZZisr(1,1)=DRR2_3(spr);

      DZZfsr(2,2)=DLL2_3(s);
      DZZfsr(1,2)=DRL3_2(s); ! note the transposition compared to ISR
      DZZfsr(2,1)=DRL2_3(s); ! note the transposition compared to ISR
      DZZfsr(1,1)=DRR2_3(s);

      call nc_ha_12_12__20_20_isrSA(p1,p2,p3,p4,p5,qel,DZZisr,ampISR)
      call nc_ha_12_12__20_20_isrSA(p3,p4,p1,p2,p5,qta,DZZfsr,ampFSR)

      do h1=1,2
        do h2=1,2
          do h3=1,2
            do h4=1,2
              do h5=1,2
                hel_amp(h1,h2,h3,h4,h5)=0d0
                do j1 = 1,2
                  do j2 = 1,2
                    hel_amp(h1,h2,h3,h4,h5)=hel_amp(h1,h2,h3,h4,h5)
     &              +cc1(j1,h1)*cc2(j2,h2)*(ampISR(j1,j2,h3,h4,h5)+ampFSR(h3,h4,j1,j2,h5))
                  enddo
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo

! phase space volume
      volum =  1d0/2048d0/pi**4*(s-spr)/s**2*betaf/betai*conhc

! amplitudes squared
      do h1=1,2
        do h2=1,2
          hard2(h1,h2) = 0d0
          do h3=1,2
            do h4=1,2
              do h5=1,2
                amp2(h1,h2,h3,h4,h5) = abs(hel_amp(h1,h2,h3,h4,h5))**2
                hard2(h1,h2) = hard2(h1,h2) + amp2(h1,h2,h3,h4,h5)
              enddo
            enddo
          enddo
        enddo
      enddo

      if (alr.eq.0) then
        hard =
     &      ((1d0+lamep)*(1d0+lamem)*hard2(1,1)
     &      +(1d0+lamep)*(1d0-lamem)*hard2(1,2)
     &      +(1d0-lamep)*(1d0+lamem)*hard2(2,1)
     &      +(1d0-lamep)*(1d0-lamem)*hard2(2,2))/4d0
      elseif (alr.eq.1) then
        hard =  hard2(1,2)-hard2(2,1)
      elseif (alr.eq.2) then
        hard =  hard2(1,2)+hard2(2,1)
      endif

      hard = hard*volum*e**6
      hard = hard*cfprime
      if (hard.ne.hard) then
        print *,"NAN hard:",s,spr
        hard = 0d0
      endif
      return
      end
