# Find the LoopTools includes and library.
#
# This module defines
# LOOPTOOLS_INCLUDE_DIR, where to locate clooptools.h file
# LOOPTOOLS_LIBRARIES, the libraries to link against to use LoopTools
# LOOPTOOLS_FOUND.  If false, you cannot build anything that requires LoopTools.
# LOOPTOOLS_LIBRARY_DIR, where to find the libooptools.a library.

message(STATUS "Looking for LoopTools")

set(LOOPTOOLS_FOUND 0)

find_path(LOOPTOOLS_INCLUDE_DIR clooptools.h
  $ENV{LOOPTOOLS_ROOT}/include
  $ENV{LOOPTOOLS}/include
  ${LOOPTOOLS_DIR}/include
  /opt/LoopTools/include
  /usr/local/include
  /usr/include
  /usr/include/LoopTools
  ~/local/include
  ~/local/LoopTools/x86_64-Linux/include
  DOC "Specify the directory containing clooptools.h."
)

#message(STATUS "Found LOOPTOOLS_INCLUDE_DIR at ${LOOPTOOLS_INCLUDE_DIR}")

#set(CMAKE_FIND_LIBRARY_SUFFIXES ".a;.so")
find_library(LOOPTOOLS_LIBRARY
    NAMES
        ooptools libooptools.a
    PATHS
        $ENV{LOOPTOOLS_ROOT}/lib
        $ENV{LOOPTOOLS}/lib
        ${LOOPTOOLS_DIR}/lib
        /opt/LoopTools/lib
        /usr/local/lib
        /usr/lib
        ~/local/lib
        ~/local/LoopTools/x86_64-Linux/lib
        ~/local/LoopTools/x86_64-Linux/lib64
    DOC "Specify the LoopTools library here."
)

#message(STATUS "Found LOOPTOOLS_LIBRARY at ${LOOPTOOLS_LIBRARY}")

if(LOOPTOOLS_INCLUDE_DIR AND LOOPTOOLS_LIBRARY)
  set(LOOPTOOLS_FOUND 1 )
  message(STATUS "Found LoopTools library at ${LOOPTOOLS_LIBRARY}")
endif()

STRING(REGEX REPLACE "(^.*)/libooptools(.a|.so)?$" "\\1" LOOPTOOLS_LIBRARY_DIR ${LOOPTOOLS_LIBRARY})
#message(STATUS "Found LOOPTOOLS_LIBRARY_DIR at ${LOOPTOOLS_LIBRARY_DIR}")

set(LOOPTOOLS_LIBRARIES ${LOOPTOOLS_LIBRARY} gfortran)

MARK_AS_ADVANCED(LOOPTOOLS_FOUND LOOPTOOLS_LIBRARY LOOPTOOLS_INCLUDE_DIR)
